#include <fcntl.h>
#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <termios.h>
#include <unistd.h>
#include <utmp.h>
#include "shadow_.h"
#include <sys/resource.h>
#include <sys/stat.h>
#include <sys/time.h>

#define STRFCPY(A,B) \
        (strncpy((A), (B), sizeof(A) - 1), (A)[sizeof(A) - 1] = '\0')

// login defines
#define FAIL_DELAY 3
#define LOGIN_PROMPT "\n%s login: "
#define NOLOGIN_FILE "/etc/nologin"
#define RETRIES 3
#define TIMEOUT 60

// sulogin defines
#define SULOGIN_PROMPT "\nType control-d to proceed with normal startup,\n(or give root password for system maintenance):"                                
#define PASSWD_FILE "/etc/passwd"
#define SHADOW_FILE "/etc/shadow"

// passwd definens

static int amroot;
static char *Prog;
struct utmp utent;
struct passwd pwent;
extern char **environ;
extern char **newenvp;
extern size_t newenvc;
static char crypt_passwd[128];

int	    my_login();
int	    my_passwd();
int	    my_sulogin();
static void alarm_handler();
char       *Basename();
static void catch();
static void check_nologin();
int         check_tty();
char       *crypt_make_salt();
int         i64c();
static int  is_my_tty();
void        login_prompt();
void        motd();
static int  new_password();
static int  pw_auth();
char       *pw_encrypt();
void        set_filesize_limit();
int         set_uid_gid();
static void update_shadow();
static void update_noshadow();
static void usage();

int main(int argc, char **argv) {
  Prog = Basename(argv[0]);
  if (Prog[0] == 'l') {
    return my_login(argc, argv);
  } else if (Prog[0] == 'p') {
    return my_passwd(argc, argv);
  } else if (Prog[0] == 's') {
    return my_sulogin(argc, argv);
  }
  puts("Unable to determine program name.\n");
  fflush(stdout);
  return(-1);
}

int my_login(int argc, char **argv) {
  char name[32];
  char tty[BUFSIZ];
  char full_tty[200];
  char fromhost[512];
  char *host = "";
  char *cp;
  char *tmp;
  int flag;
  int fflg, hflg, pflg;
  int failed;
  struct passwd  *pwd;
  struct spwd *spwd=NULL;
  char **envp = environ;
  initenv();
  name[0] = '\0';
  amroot = (getuid() == 0);
  signal(SIGALRM, alarm_handler);
  while ((flag = getopt(argc, argv, "f:h:d:p")) != EOF) {
    switch(flag) {
      case 'p':
        pflg ++;
	break;
      case 'f':
        /*
	 * username must be a seperate token
	 * (-f root, *NOT* -froot). --marekm
	 */
	if (optarg != argv[optind-1]) {
	  usage();
	}
	if (!amroot) { /* Auth bypass only if real UID is zero */
	  fprintf(stderr, "%s: -f permission denied\n", Prog);
	  exit(1);
	}
	fflg++;
	STRFCPY(name, optarg);
	break;
      case 'h':
        hflg++;
	host = optarg;
	break;
      case 'd':
        /* "-d device" ignored for compatibility */
	break;
      default:
        usage();
    }
  }
  if (!isatty(0) || !isatty(1) || !isatty(2)) {
    exit(1); /* Must be a terminal */
  }
  checkutmp(!amroot);
  STRFCPY(tty, utent.ut_line);
  if (amroot) {
    bzero(utent.ut_host, sizeof utent.ut_host);
  }
  if (hflg) {
    strncpy(utent.ut_host, host, sizeof(utent.ut_host));
    cp = host;
  }
  openlog("login", LOG_PID|LOG_CONS|LOG_NOWAIT, LOG_AUTH);
  if (pflg) {
    while (*envp) {
      addenv(*envp++, NULL);
    }
  }
  if (!pflg && (tmp = getenv("TERM"))) {
    addenv("TERM", tmp);
  }
  if (optind < argc) {
    STRFCPY(name, argv[optind]);
    optind ++;
  }
  if (optind < argc) { // Set command line variables
    set_env(argc - optind, &argv[optind]);
  }
  if (*cp) {
    snprintf(fromhost, sizeof fromhost, " on `%.100s' from `%.200s'", tty, cp);
  } else {
    snprintf(fromhost, sizeof fromhost, " on `%.100s'", tty);
  }
top:
  if (TIMEOUT > 0) {
    alarm(TIMEOUT);
  }
  environ = newenvp;
  while (1) {
    failed = 0;
    if (!name[0]) {
      login_prompt(LOGIN_PROMPT, name, sizeof name);
    }
    if (!(pwd = getpwnam(name))) {
      pwent.pw_name = name;
      pwent.pw_passwd = "!";
      pwent.pw_shell = "/bin/sh";
      fflg = 0;
      failed = 1;
    } else {
      pwent = *pwd;
    }
    spwd = NULL;
    if (pwd && ((strcmp(pwd->pw_passwd, "x") == 0) || (strcmp(pwd->pw_passwd, "*") == 0))) {
      spwd = getspnam(name);
      if (spwd) {
        pwent.pw_passwd = spwd->sp_pwdp;
      } else {
        syslog(LOG_WARNING, "no shadow password for `%s'%s\n", name, fromhost);
      }
    }
    if (pwent.pw_passwd[0] == '!' || pwent.pw_passwd[0] == '*') {
      failed = 1;
    }
    if (fflg) {
      fflg--;
      goto auth_ok;
    }
    if ((pwent.pw_uid == 0) && (check_tty(tty))) {
      failed = 1;
    }
    if (pwent.pw_passwd[0] == '\0') {
      goto auth_ok;
    }
    if (pw_auth(pwent.pw_passwd, name) == 0) {
      goto auth_ok;
    }
    syslog(LOG_WARNING, "invalid password for `%s'%s\n", pwd ? name : "UNKNOWN", fromhost);
    failed = 1;
auth_ok:
    if (!failed) {
      break;
    }
    if (pwent.pw_passwd[0] == '\0') {
      pw_auth("!", name);
    }
    bzero(name, sizeof name);
    sleep(FAIL_DELAY);
    puts("Login incorrect");
  }
  (void) alarm(0);
  check_nologin();
  if (getenv("IFS")) {
    addenv("IFS= \t\n", NULL);
  }
  setutmp(name, tty);
  if (*tty != '/') {
    snprintf(full_tty, sizeof full_tty, "/dev/%s", tty);
  } else {
    strncpy(full_tty, tty, sizeof full_tty);
  }
  if(! is_my_tty(full_tty)) {
    syslog(LOG_ERR, "unable to determine TTY name, got %s\n", full_tty);
  }
  if (chown(full_tty, pwent.pw_uid, pwent.pw_gid) ||
      chmod(full_tty, 0600)) {
    perror("unable to change TTY");
    syslog(LOG_ERR, "unable to change tty `%s' for user `%s'\n", tty, pwent.pw_name);
    closelog();
    exit(1);
  }
  if (set_uid_gid() != 0) {
    exit(1);
  }
  setup_env(&pwent);
  motd();
  signal(SIGINT, SIG_DFL);        /* default interrupt signal */
  signal(SIGQUIT, SIG_DFL);       /* default quit signal */
  signal(SIGTERM, SIG_DFL);       /* default terminate signal */
  signal(SIGALRM, SIG_DFL);       /* default alarm signal */
  signal(SIGHUP, SIG_DFL);        /* added this.  --marekm */
  endpwent();                    /* stop access to password file */
  endgrent();                    /* stop access to group file */
  endspent();                    /* stop access to shadow passwd file */
//  endsgent();                    /* stop access to shadow group file */
  if (pwent.pw_uid == 0) {
    syslog(LOG_INFO, "root login from `%s'\n", fromhost);
  }
  closelog ();
  shell (pwent.pw_shell, (char *) 0); /* exec the shell finally. */
  /*NOTREACHED*/
  return (0);
}

extern char *crypt_make_salt();

int my_passwd(int argc, char **argv) {
  char *cp;
  char *name;
  char *myname;
  int flag;
  int lflg = 0;    /* -l - lock account */
  int uflg = 0;    /* -u - unlock account */
  int dflg = 0;    /* -d - delete password */
  const struct passwd *pw;
  const struct spwd *sp;
  amroot = (getuid() == 0);
  openlog("passwd", LOG_PID|LOG_CONS|LOG_NOWAIT, LOG_AUTH);
#define FLAGS "dlu"
  while ((flag = getopt(argc, argv, "dlu")) != EOF) {
    switch(flag) {
      case 'd':
	dflg++;
	break;
      case 'l':
	lflg++;
	break;
      case 'u':
	uflg++;
	break;
     default:
       usage();
    }
  }
  pw = (struct passwd *)get_my_pwent();
  if (!pw) {
    fprintf(stderr, "%s: Cannot determine your user name.\n", Prog);
    exit(1);
  }
  myname = (char *)xstrdup(pw->pw_name);
  if (optind < argc) {
    name = argv[optind];
  } else {
    name = myname;
  }
  if ((lflg || uflg || dflg) && (optind >= argc || !amroot)) {
    usage();
  }
  pw = getpwnam(name);
  if (!pw) {
    fprintf(stderr, "%s: Unknown user %s\n", Prog, name);
    exit(1);
  }
  if (!amroot && pw->pw_uid != getuid()) {
    fprintf(stderr, "%s: Permission denied.\n", Prog);
    syslog(LOG_WARNING, "can't change pwd for `%s'", name);
    closelog();
    exit(1);
  }
  sp = getspnam(name);
  if (!sp) {
    sp = (struct spwd *)pwd_to_spwd(pw);
  }
  cp = sp->sp_pwdp;
  if (!(dflg || lflg || uflg)) {
    STRFCPY(crypt_passwd, cp);
  }
  if (!amroot) {
    if (sp->sp_pwdp[0] == '!') {
      fprintf(stderr, "The password for `%s' cannot be changed.\n", sp->sp_namp);
      syslog(LOG_WARNING, "password locked for `%s'", sp->sp_namp);
      closelog();
      exit(1);
    }
  }
  printf("Changing password for %s\n", name);
  if (new_password(pw)) {
    fprintf(stderr, "The password for %s is unchanged.\n", name);
    closelog();
    exit(1);
  }
  set_filesize_limit(30000);
  signal(SIGHUP, SIG_IGN);
  signal(SIGINT, SIG_IGN);
  signal(SIGQUIT, SIG_IGN);
  umask(077);
  if (setuid(0)) {
    fprintf(stderr, "Cannot change ID to root.\n");
    syslog(LOG_ERR, "can't setuid(0)");
    closelog();
    exit(1);
  }
  if (access(SHADOW_FILE, F_OK) == 0){
    update_shadow();
  } else {
    update_noshadow();
  }
  syslog(LOG_INFO, "password for `%s' changed by user `%s'", name, myname);
  closelog();
  printf("Password changed.\n");
  exit(0);
}

int my_sulogin(int argc, char **argv) {
  char *cp;
  char *device = (char *)0;
  char **envp = environ;
  char *name = "root";
  int timeout = 0;
  static char pass[BUFSIZ];
  struct termios termio;
  struct passwd  *pwd;
  struct spwd *spwd=NULL;
  tcgetattr (0, &termio);                                                 
  termio.c_iflag |= (ICRNL|IXON);                                         
  termio.c_oflag |= (CREAD);                                              
  termio.c_lflag |= (ECHO|ECHOE|ECHOK|ICANON|ISIG);                       
  tcsetattr (0, TCSANOW, &termio);                                        
  openlog ("sulogin", LOG_PID|LOG_CONS|LOG_NOWAIT, LOG_AUTH);
  initenv();
  if (argc > 1) {
    if (strncmp(argv[1], "-t", 2) == 0) {
      if (strcmp(argv[1], "-t") == 0) {
	if (argc > 2) {
	  timeout = atoi(argv[2]);
	  if (argc > 3) {
	    device = argv[3];
	  }
	}
      } else {
	if (argc > 2) {
	  device = argv[2];
	}
      }
    } else {
      device = argv[1];
    }
    if (device) {
      close (0);
      close (1);
      close (2);
      if (open(device, O_RDWR) >= 0) {
	dup(0);
	dup(0);
      } else {
	syslog (LOG_WARNING, "cannot open %s\n", device);
	closelog();
	exit(1);
      }
    }
  }
  if (access(PASSWD_FILE, 0) == -1) {
    printf("No password file\n");
    syslog(LOG_WARNING, "No password file\n");
    closelog();
    exit(1);
  }
  if (!isatty(0) || !isatty(1) || !isatty(2)) {
    closelog();
    exit(1);
  }
  while (*envp) {
    addenv(*envp++, NULL);
  }
  signal (SIGALRM, catch);
  alarm(timeout);
  if (!(pwd = getpwnam(name))) {
    printf("No password entry for `root'\n");
    syslog(LOG_WARNING, "No password entry for `root'\n");
    closelog();
    exit(1);
  }
  pwent = *pwd;
  spwd = NULL;
  if (pwd &&  ((strcmp(pwd->pw_passwd, "x") == 0) || (strcmp(pwd->pw_passwd, "*") == 0))) {
      endspent();
      spwd = getspnam(name);
      if (spwd) {
        pwent.pw_passwd = spwd->sp_pwdp;
      }
    }
  while (1) {
    cp = getpass(SULOGIN_PROMPT);
    if (!cp || !*cp) {
      syslog(LOG_INFO, "Normal startup\n");
      closelog();
      puts("\n");
      exit(0);
    } else {
      STRFCPY(pass, (cp));
      bzero(cp, strlen(cp));
    }
    if (strcmp(pw_encrypt(pass, pwent.pw_passwd), pwent.pw_passwd) == 0) {
      break;
    }
    syslog(LOG_WARNING, "Incorrect root password\n");
    sleep(FAIL_DELAY);
    puts("Login incorrect");
  }
  bzero(pass, strlen(pass));
  alarm(0);
  signal(SIGALRM, SIG_DFL);
  environ = newenvp;
  puts("Entering System Maintenance Mode\n");
  syslog(LOG_INFO, "System Maintenance Mode\n");
  closelog();
  shell(pwent.pw_shell, (char *)0);
  return (0);
}

static void alarm_handler() {
  fprintf(stderr, "\nLogin timed out after %d seconds.\n", TIMEOUT);
  exit(0);
}

char *Basename(char *str) {
  char *cp = strrchr(str, '/');
  return cp ? cp+1 : str;
}

static void catch() {
  exit(1);
}

static void check_nologin() {
  if (access(NOLOGIN_FILE, F_OK) == 0) {
    FILE    *nlfp;                                                  
    int     c;                                                      
    if ((nlfp = fopen (NOLOGIN_FILE, "r"))) {                              
      while ((c = getc (nlfp)) != EOF) {                      
        if (c == '\n')                                  
          putchar ('\r');                         
        putchar (c);                                    
      }                                                       
      fflush (stdout);                                        
      fclose (nlfp);                                          
    } else {
      printf("\r\nSystem closed for routine maintenance.\r\n");
    }
    if (pwent.pw_uid != 0) {
      closelog();
      exit(0);
    }
    printf("\r\n[Disconnect bypassed -- root login allowed.]\r\n");
  }
}

int check_tty(const char *tty) {
  FILE *fp;
  int i;
  char buf[BUFSIZ];
  if ((fp = fopen("/etc/securetty", "r")) == NULL) {
    syslog(LOG_WARNING, "cannot open securetty file.\n");
    return 0;
  }
  while (fgets(buf, sizeof(buf), fp) != NULL) {
    for (i = strlen(buf) - 1; i >= 0; --i) {
      if (!isspace(buf[i])) {
	break;
      }
    }
    buf[++i] = '\0';
    if (buf[0] == '\0' || buf[0] == '#') {
      continue;
    }
    if (strcmp(buf, tty) == 0) {
      fclose(fp);
      return 0;
    }
  }
  fclose(fp);
  return 1;
}

char *crypt_make_salt()
{
  time_t now;
  static unsigned long x;
  static char result[3];
  time(&now);
  x += now + getpid() + clock();
  result[0] = i64c(((x >> 18) ^ (x >> 6)) & 077);
  result[1] = i64c(((x >> 12) ^ x) & 077);
  result[2] = '\0';
  return result;
}

int i64c(int i) {
  if (i <= 0)
    return ('.');
  if (i == 1)
    return ('/');
  if (i >= 2 && i < 12)
    return ('0' - 2 + i);
  if (i >= 12 && i < 38)
    return ('A' - 12 + i);
  if (i >= 38 && i < 63)
    return ('a' - 38 + i);
  return ('z');
}

static int is_my_tty(const char *tty) {
  struct stat by_name, by_fd;
  if (stat(tty, &by_name) || fstat(0, &by_fd)) {
    return 0;
  }
  if (by_name.st_rdev != by_fd.st_rdev) {
    return 0;
  } else {
    return 1;
  }
}

void login_prompt(const char *prompt, char *name, int namesize) {
  char buf[1024];
  char *cp;
  int i;
  void (*sigquit)();
  sigquit = signal(SIGQUIT, _exit);
  if (prompt) {
    gethostname(buf, sizeof buf);
    printf(prompt, buf);
    fflush(stdout);
  }
  bzero(buf, sizeof buf);
  if (fgets(buf, sizeof buf, stdin) != buf) {
    exit(1);
  }
  cp = strchr(buf, '\n');
  if (!cp) {
    exit(1);
  }
  *cp = '\0';

  for (cp = buf; *cp == ' ' || *cp == '\t'; cp++);
  for (i = 0; i < namesize - 1 && isgraph(*cp); name[i++] = *cp++);
  while (isgraph(*cp)) {
    cp++;
  }
  if (*cp) {
    cp++;
  }
  name[i] = '\0';
  signal(SIGQUIT, sigquit);
}

void motd() {
  FILE *fp;
  register int c;
  if ((fp = fopen("/etc/motd", "r")) != NULL) {
    while ((c = getc(fp)) != EOF) {
      putchar(c);
    }
    fclose(fp);
  }
}

static int new_password(const struct passwd *pw) {
  char *clear;
  char *cipher;
  char *cp;
  char orig[200];
  char pass[200];
  int warned = 0;
  if (!amroot && crypt_passwd[0]) {
    if (!(clear = getpass("Old password:"))) {
      return -1;
    }
    cipher = pw_encrypt(clear, crypt_passwd);
    if (strcmp(cipher, crypt_passwd) != 0) {
      syslog(LOG_WARNING, "incorrect password for `%s'", pw->pw_name);
      sleep(FAIL_DELAY);
      fprintf(stderr, "Incorrect password.\n");
      return -1;
    }
    STRFCPY(orig, clear);
    bzero(clear, strlen(clear));
    bzero(cipher, strlen(cipher));
  } else {
    orig[0] = '\0';
  }
  if (!(cp = getpass("Enter the new password (minimum of 5, maximum of 8 characters)\nPlease use a combination of upper and lower case letters and numbers.\n"))) {
    bzero(orig, sizeof orig);
    return -1;
  }
  STRFCPY(pass, cp);
  bzero(cp, strlen(cp));
  if (!obscure(orig, pass, pw)) {
    if (amroot) {
      printf("\nWarning: weak password (continuing).\n");
    } else {
      return -1;
    }
  }
  if (!(cp = getpass("Re-enter new password:"))) {
    bzero(orig, sizeof orig);
    return -1;
  }
  if (strcmp(cp, pass)) {
    fprintf(stderr, "Passwords do not match.\n");
    return -1;
  }
  bzero(cp, strlen(cp));
  bzero(orig, sizeof(orig));
  cp = pw_encrypt(pass, crypt_make_salt());
  bzero(pass, sizeof pass);
  STRFCPY(crypt_passwd, cp);
  return 0;
}

static int  pw_auth(const char *cipher, const char *user) {
  char prompt[1024];
  char *clear = NULL;
  const char *cp;
  int retval;
  if (cipher == (char *) 0 || *cipher == '\0') {
    return 0;
  }
  clear = getpass("Password: ");
  if (!clear) {
    static char c[1];
    c[0] = '\0';
    clear = c;
  }
  retval = strcmp(pw_encrypt(clear, cipher), cipher);
  bzero(clear, strlen(clear));
  return retval;
}

char *pw_encrypt(const char *clear, const char *salt) {
  static char cipher[128];
  char *cp;

  if (strncmp(salt, "$1$", 3) == 0) {
//    return md5_crypt(clear, salt);
  }
  cp = (char *)crypt(clear, salt);
  if (strlen(cp) != 13) {
    return cp; /* nonstandard crypt, better bail */
  }
  strcpy(cipher, cp);
  return cipher;
}

void set_filesize_limit(int blocks) {
  struct rlimit rlimit_fsize;
  rlimit_fsize.rlim_cur = rlimit_fsize.rlim_max = 512L * blocks;
  setrlimit(RLIMIT_FSIZE, &rlimit_fsize);
}

int set_uid_gid() {
  if (initgroups(pwent.pw_name, pwent.pw_gid) == -1) {
    perror("initgroups");
    syslog(LOG_ERR, "initgroups failed for user `%s': %,\n", pwent.pw_name);
    closelog();
    return -1;
  }
  if (setgid(pwent.pw_gid) == -1) {
    perror("setgid");
    syslog(LOG_ERR, "bad group ID `%d' for user `%s': %m\n", pwent.pw_gid, pwent.pw_name);
    closelog();
    return -1;
  }
  if (setuid(pwent.pw_uid)) {
    perror("setuid");
    syslog(LOG_ERR, "bad user ID `%d' for user `%s': %m\n", pwent.pw_uid, pwent.pw_name);
    closelog();
    return -1;
  }
  return 0;
}

static void update_shadow() {
}

static void usage() {
  fprintf(stderr, "usage: %s [name]\n", Prog);
  exit(1);
}
